#!/usr/bin/env python
# coding: utf-8

import json
import os
import re
from pathlib import Path

from tabulate import tabulate


def status() -> str:
    """Get the status of the agents you created."""
    lagents_rc = f'{Path.home()}/.lpyrc'
    r, g, c, y, R = '\033[31m', '\033[32m', '\033[36m', '\033[33m', '\033[39m'

    if not Path(lagents_rc).exists():
        return f'{r}Could not find any custom agents...{R}'
    with open(lagents_rc, 'r') as j:
        agents = json.load(j)
        prefix = agents['prefix']

    table = []

    for agent in agents['agents']:
        out = os.popen(f'launchctl list {agent}').read()
        match = re.search(r'.+"PID".+', out)
        if match:
            pid = int(match[0].strip()[:-1].split('= ')[1])
            pid = f'{g}{pid}{R}'
        else:
            pid = f'\033[40m{r}DOWN{R}\033[49m'
        plist_path = f'{prefix}/{agent}.plist'.replace(str(Path.home()), '~')
        plist_path = f'{c}{plist_path}{R}'
        row = (plist_path, pid)
        table.append(row)

    headers = [f'{r}{x}{R}' for x in ['PATH', 'PID']]

    t = tabulate(table,
                 headers=headers,
                 tablefmt='pretty',
                 stralign='left',
                 showindex=True)
    return t



#!/usr/bin/env python
# coding: utf-8

import json
import shlex
import signal
import sys
import textwrap
from pathlib import Path
from typing import Optional


def keyboard_interrupt_handler(sig: int, _) -> None:
    """Handle keyboard interrupt."""
    print(f'\nKeyboardInterrupt (id: {sig}) has been caught...')
    print('Terminating the session gracefully...')
    sys.exit(1)


def create(noninteractive: bool = False,
           agent_name: Optional[str] = None,
           exec_bin: Optional[str] = None,
           cmd_args: Optional[str] = None) -> None:
    """Create a new launchctl agent."""
    signal.signal(signal.SIGINT, keyboard_interrupt_handler)

    HOME = Path.home()

    lagents_rc = f'{Path.home()}/.lpyrc'

    if not Path(lagents_rc).exists():
        with open(lagents_rc, 'w') as j:
            LAGENTS_PREFIX = f'{HOME}/Library/LaunchAgents'
            DOMAIN = 'local'
            L_PATH = f'{HOME}/Library/Logs'
            json.dump(
                {
                    'prefix': LAGENTS_PREFIX,
                    'domain': DOMAIN,
                    'logs': L_PATH,
                    'agents': []
                }, j)
    else:
        with open(lagents_rc) as j:
            lagents_rc_content = json.load(j)
            LAGENTS_PREFIX = lagents_rc_content['prefix']
            DOMAIN = lagents_rc_content['domain']
            L_PATH = lagents_rc_content['logs']

    if not noninteractive:
        agent_name = input('Agent Name (CamelCase): ')
        exec_bin = input('Executable binary full path: ')
        cmd_args = input('Program arguments: ')

    program_args = ''
    for n, arg in enumerate(shlex.split(cmd_args)):
        if n == 0:
            program_args += f'\t\t<string>{arg}</string>\n'
        else:
            program_args += f'\t\t\t<string>{arg}</string>\n'

    plist_content = f'''\
    <?xml version="1.0" encoding="UTF-8"?>
    <!DOCTYPE plist PUBLIC "-//Apple//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd"> <plist version="1.0"> 
    <dict>
        <key>Label</key>
        <string>com.{DOMAIN}.{agent_name}</string> 

        <key>ProgramArguments</key>
        <array>
            <string>{exec_bin}</string>
    {program_args.rstrip()}
        </array>

        <key>RunAtLoad</key>
        <true/>

        <key>KeepAlive</key>
        <true/>

        <key>StandardErrorPath</key>
        <string>{L_PATH}/com.{DOMAIN}/com.{DOMAIN}.{agent_name}.err</string>

        <key>StandardOutPath</key>
        <string>{L_PATH}/com.{DOMAIN}/com.{DOMAIN}.{agent_name}.out</string>
    </dict>
    </plist>
    '''  # noqa: E501
    plist_content = textwrap.dedent(plist_content)

    Path(f'{L_PATH}/com.{DOMAIN}').mkdir(exist_ok=True)

    plist_fpath = f'{LAGENTS_PREFIX}/com.{DOMAIN}.{agent_name}.plist'

    print('-' * 80)
    print(plist_content)
    print('-' * 80)

    ans = input('\n\nConfirm? (y/N) ')

    if ans.lower() not in ['y', 'yes']:
        sys.exit(1)

    with open(plist_fpath, 'w') as f:
        f.write(plist_content)

    with open(lagents_rc, 'r+') as j:
        agents = json.load(j)
        entry = f'com.{DOMAIN}.{agent_name}'
        if entry not in agents['agents']:
            agents['agents'].append(entry)
            j.seek(0)
            json.dump(agents, j, indent=4)

    print('Run:')
    print(f'    sudo chown root:wheel {plist_fpath} && '
          f'sudo chmod o-w {plist_fpath} && '
          f'launchctl load {plist_fpath} && '
          f'launchctl list {Path(plist_fpath).stem} | grep \'"PID"\'')



#!/usr/bin/env python
# coding: utf-8

import argparse

from launchctl_py.create import create
from launchctl_py.status import status

from launchctl_py.create import create
from launchctl_py.status import status


def _opts() -> argparse.ArgumentParser:
    parser = argparse.ArgumentParser()
    parser.add_argument("-c",
                        "--create",
                        help="Create a new launchctl agent",
                        action="store_true")
    parser.add_argument("-s",
                        "--status",
                        help="Get the status of the agents you created",
                        action="store_true")
    return parser


def main():
    opts = _opts().parse_args()
    if opts.create:
        create()
    elif opts.status:
        print(status())
    else:
        _opts().print_help()


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['launchctl_py']

package_data = \
{'': ['*']}

install_requires = \
['tabulate>=0.8.9']

entry_points = \
{'console_scripts': ['launchctl_py = launchctl_py:cli.main',
                     'launchctl_python = launchctl_py:cli.main',
                     'lpy = launchctl_py:cli.main']}

setup_kwargs = {
    'name': 'launchctl-py',
    'version': '0.1.2',
    'description': 'Quickly create simple background services (i.e., launchctl agents) for macOS.',
    'long_description': '# Launchctl-Py\n\n🚀 Quickly create _super basic_ background services (i.e., launchctl agents) for macOS in Python.\n\n[![Supported Python versions](https://img.shields.io/badge/Python-%3E=3.7-blue.svg?logo=python)](https://www.python.org/downloads/) [![PEP8](https://img.shields.io/badge/Code%20style-PEP%208-orange.svg?logo=python)](https://www.python.org/dev/peps/pep-0008/) ![platform](https://img.shields.io/badge/Platform-macOS-green.svg?logo=apple)\n\n\n## Requirements\n- 🐍 [python>=3.7](https://www.python.org/downloads/)\n\n## ⬇️ Installation\n\n```sh\npip install launchctl-py\n```\n\n## ⌨️ Usage\n\n```\nusage: lpy [-h] [-c] [-s]\n\noptional arguments:\n  -h, --help    show this help message and exit\n  -c, --create  Create a new launchctl agent\n  -s, --status  Get the status of the agents you created\n```\n\n\n## 💡 Misc.\n\nOptional: if you want the domain name of your agents to be something other than `local` (default), run:\n```sh\nlaunchctl setenv DEFAULT_DOMAIN "ReplaceMe"\necho \'setenv DEFAULT_DOMAIN ReplaceMe\' >> ~/.conf.launchd\n```\n\n---\n\n...\\\n**👷 Additional features are under development...**\\\n...\n',
    'author': 'Alyetama',
    'author_email': 'malyetama@pm.me',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
